@@ -22,7 +22,7 @@ Net::SSLeay::ASN1_GENERALIZEDTIME class in L<SPVM> represents L<ASN1_GENERALIZED
 
 C<method DESTROY : void ();>
 
-Calls native L<ASN1_STRING_free|https://docs.openssl.org/master/man3/ASN1_STRING_new/> function given the pointer value of the instance if C<no_free> flag of the instance is not a true value.
+Calls native L<ASN1_GENERALIZEDTIME_free|https://github.com/google/boringssl/blob/master/include/openssl/asn1.h> function given the pointer value of the instance if C<no_free> flag of the instance is not a true value.
 
 =head1 See Also
 
