@@ -22,7 +22,7 @@ Net::SSLeay::OCSP_CERTID class in L<SPVM> represents L<OCSP_CERTID|https://docs.
 
 C<method DESTROY : void ();>
 
-Frees native L<OCSP_CERTID|https://docs.openssl.org/1.1.1/man3/OCSP_cert_to_id/> object by calling native L<OCSP_CERTID_free|https://docs.openssl.org/1.1.1/man3/OCSP_cert_to_id/> function if C<no_free> flag of the instance is not a true value.
+Calls native L<OCSP_CERTID_free|https://docs.openssl.org/1.1.1/man3/OCSP_cert_to_id/> function given the pointer value of the instance if C<no_free> flag of the instance is not a true value.
 
 =head1 See Also
 
